@@ -1,10 +1,17 @@
 import sqlite3
 import threading
 from pathlib import Path
-from typing import Iterator, Optional, Tuple
+from typing import Iterator, Optional, Union
 from uuid import uuid4
 
-from ..trie import AbstractTrie, Change, ShortKeyError, TrieKey, TrieNode
+from ..trie import (
+    AbstractTrie,
+    Change,
+    ShortKeyError,
+    TrieKey,
+    TrieNode,
+    TrieStep,
+)
 
 # NOTE: seems like "named" doesn't work without changing this global var,
 # so unfortunately we have to stick with qmark.
@@ -198,32 +205,24 @@ class SQLiteTrie(AbstractTrie):
             """
         ).fetchone()["count"]
 
-    def shortest_prefix(
-        self, key: TrieKey
-    ) -> Tuple[Optional[TrieKey], Optional[bytes]]:
-        skey: TrieKey = ()
-        value = None
+    def prefixes(self, key: TrieKey) -> Iterator[TrieStep]:
         for row in self._traverse(key):
-            skey = (*skey, row["name"])  # type: ignore
-            value = row["value"]
-            if value is not None:
-                break
-
-        return skey, value
-
-    def longest_prefix(self, key) -> Tuple[Optional[TrieKey], Optional[bytes]]:
-        rows = self._traverse(key)
-        lkey: TrieKey = ()
-        value = None
-        for idx, row in enumerate(reversed(rows)):
-            if row["value"] is None:
+            if not row["has_value"]:
                 continue
-            lkey = tuple(  # type: ignore
-                row["name"] for row in rows[: len(rows) - idx]
+
+            yield (
+                tuple(row["path"].split("/")),  # type: ignore
+                row["value"],
             )
-            value = row["value"]
-            break
-        return lkey, value
+
+    def shortest_prefix(self, key: TrieKey) -> Optional[TrieStep]:
+        return next(self.prefixes(key), None)
+
+    def longest_prefix(self, key) -> Optional[TrieStep]:
+        ret = None
+        for step in self.prefixes(key):
+            ret = step
+        return ret
 
     def view(  # type: ignore
         self,
@@ -265,10 +264,18 @@ class SQLiteTrie(AbstractTrie):
         except KeyError:
             return False
 
-    def ls(self, key: TrieKey) -> Iterator[TrieKey]:
-        yield from (  # type: ignore
-            (*key, row["name"]) for row in self._get_children(key)
-        )
+    def ls(
+        self, key: TrieKey, with_values: Optional[bool] = False
+    ) -> Iterator[Union[TrieKey, TrieNode]]:
+        if with_values:
+            yield from (  # type: ignore
+                ((*key, row["name"]), row["value"])
+                for row in self._get_children(key)
+            )
+        else:
+            yield from (  # type: ignore
+                (*key, row["name"]) for row in self._get_children(key)
+            )
 
     def traverse(self, node_factory, prefix=None):
         key = prefix or ()
